@@ -16,17 +16,22 @@ import json
 from datetime import timedelta
 from typing import Any, Dict, List, Optional
 
-from robotlibcore import keyword  # type: ignore
+from robot.utils import get_link_path  # type: ignore
 
 from ..assertion_engine import verify_assertion, with_assertion_polling
 from ..base import LibraryComponent
 from ..generated.playwright_pb2 import Request
 from ..utils import (
     ColorScheme,
+    GeoLocation,
+    HttpCredentials,
+    Proxy,
     SelectionType,
     SupportedBrowsers,
     ViewportDimensions,
+    convert_typed_dict,
     find_by_id,
+    keyword,
     locals_to_params,
     logger,
 )
@@ -50,7 +55,7 @@ class PlaywrightState(LibraryComponent):
         if context != "CURRENT":
             self.switch_context(context)
 
-    @keyword(tags=["Setter", "BrowserControl"])
+    @keyword(tags=("Setter", "BrowserControl"))
     def open_browser(
         self,
         url: Optional[str] = None,
@@ -65,9 +70,9 @@ class PlaywrightState(LibraryComponent):
         Creates a new browser, context and page with specified settings.
             Only supports some of the settings Create _ Keywords do
 
-        ``url`` <str> Navigates to URL if provided. Defaults to None.
+        ``url`` Navigates to URL if provided. Defaults to None.
 
-        ``browser`` < ``firefox`` | ``chromium`` | ``webkit`` > Specifies which browser to use. The
+        ``browser`` Specifies which browser to use. The
         supported browsers are listed in the table below. The browser names
         are case-sensitive.
         |   = Value =     |        = Name(s) =                                   |
@@ -75,9 +80,9 @@ class PlaywrightState(LibraryComponent):
         | chromium        | [https://www.chromium.org/Home|Chromium]             |
         | webkit          | [https://webkit.org/|webkit]                         |
 
-        ``headless`` <bool> If set to False, a GUI is provided otherwise it is hidden. Defaults to False.
+        ``headless`` If set to False, a GUI is provided otherwise it is hidden. Defaults to False.
 
-        ``pause_on_failure`` <bool> Stop execution when failure detected and leave browser open. Defaults to True.
+        ``pause_on_failure`` Stop execution when failure detected and leave browser open. Defaults to True.
         """
         logger.warn(
             "Open Browser is for quick experimentation and debugging only. Use New Page for production."
@@ -88,7 +93,7 @@ class PlaywrightState(LibraryComponent):
         if pause_on_failure:
             self.library._pause_on_failure.add(browser_id)
 
-    @keyword(tags=["Setter", "BrowserControl"])
+    @keyword(tags=("Setter", "BrowserControl"))
     def close_browser(self, browser: str = "CURRENT"):
         """Closes the current browser. Activated browser is set to first active browser.
         Closes all context and pages belonging to this browser.
@@ -112,7 +117,7 @@ class PlaywrightState(LibraryComponent):
             self.library._pause_on_failure.discard(closed_browser_id)
             logger.info(response.log)
 
-    @keyword(tags=["Setter", "BrowserControl"])
+    @keyword(tags=("Setter", "BrowserControl"))
     def close_context(self, context: str = "CURRENT", browser: str = "CURRENT"):
         """Closes a Context. Activated context is set to first active context.
         Closes pages belonging to this context.
@@ -135,6 +140,7 @@ class PlaywrightState(LibraryComponent):
     def _close_context(self, contexts):
         with self.playwright.grpc_channel() as stub:
             for context in contexts:
+                self.context_cache.remove(context["id"])
                 self.switch_context(context["id"])
                 response = stub.CloseContext(Request().Empty())
                 logger.info(response.log)
@@ -161,7 +167,7 @@ class PlaywrightState(LibraryComponent):
             browser_ids = [browser]
         return [find_by_id(browser_id, catalog) for browser_id in browser_ids]
 
-    @keyword(tags=["Setter", "BrowserControl"])
+    @keyword(tags=("Setter", "BrowserControl"))
     def close_page(
         self, page: str = "CURRENT", context: str = "CURRENT", browser: str = "CURRENT"
     ):
@@ -222,7 +228,7 @@ class PlaywrightState(LibraryComponent):
                         response = stub.ClosePage(Request().Empty())
                         logger.info(response.log)
 
-    @keyword(tags=["Setter", "BrowserControl"])
+    @keyword(tags=("Setter", "BrowserControl"))
     def new_browser(
         self,
         browser: SupportedBrowsers = SupportedBrowsers.chromium,
@@ -230,7 +236,7 @@ class PlaywrightState(LibraryComponent):
         executablePath: Optional[str] = None,
         args: Optional[List[str]] = None,
         ignoreDefaultArgs: Optional[List[str]] = None,
-        proxy: Optional[Dict] = None,
+        proxy: Optional[Proxy] = None,
         downloadsPath: Optional[str] = None,
         handleSIGINT: bool = True,
         handleSIGTERM: bool = True,
@@ -246,65 +252,70 @@ class PlaywrightState(LibraryComponent):
 
         Returns a stable identifier for the created browser.
 
-        ``browser`` <chromium|firefox|webkit> Opens the specified browser. Defaults to chromium.
+        ``browser`` Opens the specified browser. Defaults to chromium.
 
-        ``headless`` <bool> Set to False if you want a GUI. Defaults to False.
+        ``headless`` Set to False if you want a GUI. Defaults to False.
 
-        ``executablePath`` <str> Path to a browser executable to run instead of the bundled one.
+        ``executablePath`` Path to a browser executable to run instead of the bundled one.
         If executablePath is a relative path, then it is resolved relative to current working
         directory. Note that Playwright only works with the bundled Chromium, Firefox or
         WebKit, use at your own risk. Defaults to None.
 
-        ``args`` <List<str>> Additional arguments to pass to the browser instance. The list of
+        ``args`` Additional arguments to pass to the browser instance. The list of
         Chromium flags can be found [http://peter.sh/experiments/chromium-command-line-switches/ | here].
         Defaults to None.
 
-        ``ignoreDefaultArgs`` <List<str>> If an array is given, then filters out the given default arguments.
+        ``ignoreDefaultArgs`` If an array is given, then filters out the given default arguments.
         Defaults to None.
 
-        ``proxy`` <Dict> Network proxy settings.
+        ``proxy`` Network proxy settings.
         - server <string> Proxy to be used for all requests. HTTP and SOCKS proxies are supported, for example ``http://myproxy.com:3128`` or ``socks5://myproxy.com:3128``. Short form ``myproxy.com:3128`` is considered an HTTP proxy.
         - bypass <string> Optional coma-separated domains to bypass proxy, for example ``".com, chromium.org, .domain.com"``.
         - username <string> Optional username to use if HTTP proxy requires authentication.
         - password <string> Optional password to use if HTTP proxy requires authentication.
 
-        ``downloadsPath`` <str> If specified, accepted downloads are downloaded into this folder.
+        ``downloadsPath`` If specified, accepted downloads are downloaded into this folder.
         Otherwise, temporary folder is created and is deleted when browser is closed.
 
-        ``handleSIGINT`` <bool> Close the browser process on Ctrl-C. Defaults to True.
+        ``handleSIGINT`` Close the browser process on Ctrl-C. Defaults to True.
 
-        ``handleSIGTERM`` <bool> Close the browser process on SIGTERM. Defaults to True.
+        ``handleSIGTERM`` Close the browser process on SIGTERM. Defaults to True.
 
-        ``handleSIGHUP`` <bool> Close the browser process on SIGHUP. Defaults to True.
+        ``handleSIGHUP`` Close the browser process on SIGHUP. Defaults to True.
 
-        ``timeout`` <int> Maximum time in milliseconds to wait for the browser instance to start.
+        ``timeout`` Maximum time in milliseconds to wait for the browser instance to start.
         Defaults to 30000 (30 seconds). Pass 0 to disable timeout.
 
         ``env`` <Dict<str, str|int|bool>> Specify environment variables that will
         be visible to the browser. Defaults to None.
 
-        ``devtools`` <bool> Chromium-only Whether to auto-open a Developer Tools panel for each tab.
+        ``devtools`` Chromium-only Whether to auto-open a Developer Tools panel for each tab.
         If this option is true, the headless option will be set false.
 
-        ``slowMo`` <int> Slows down Playwright operations by the specified amount of milliseconds.
+        ``slowMo`` Slows down Playwright operations by the specified amount of milliseconds.
         Useful so that you can see what is going on. Defaults to no delay.
         """
         params = locals_to_params(locals())
+        params = convert_typed_dict(self.new_context.__annotations__, params)
         if timeout:
             params["timeout"] = self.convert_timeout(timeout)
         params["slowMo"] = self.convert_timeout(slowMo)
+
+        browser_path = self.library.external_browser_executable.get(browser)
+        if browser_path:
+            params["executablePath"] = browser_path
+
         options = json.dumps(params, default=str)
         logger.info(options)
 
         with self.playwright.grpc_channel() as stub:
-
             response = stub.NewBrowser(
                 Request().Browser(browser=browser.name, rawOptions=options)
             )
             logger.info(response.log)
             return response.body
 
-    @keyword(tags=["Setter", "BrowserControl"])
+    @keyword(tags=("Setter", "BrowserControl"))
     def new_context(
         self,
         acceptDownloads: bool = False,
@@ -317,15 +328,18 @@ class PlaywrightState(LibraryComponent):
         hasTouch: bool = False,
         javaScriptEnabled: bool = True,
         timezoneId: Optional[str] = None,
-        geolocation: Optional[Dict] = None,
+        geolocation: Optional[GeoLocation] = None,
         locale: Optional[str] = None,
         permissions: Optional[List[str]] = None,
         extraHTTPHeaders: Optional[Dict[str, str]] = None,
         offline: bool = False,
-        httpCredentials: Optional[Dict] = None,
+        httpCredentials: Optional[HttpCredentials] = None,
         colorScheme: Optional[ColorScheme] = None,
+        proxy: Optional[Proxy] = None,
+        videosPath: Optional[str] = None,
+        videoSize: Optional[ViewportDimensions] = None,
+        defaultBrowserType: Optional[SupportedBrowsers] = None,
         hideRfBrowser: bool = False,
-        defaultBrowserType: Optional[str] = None,
     ) -> str:
         """Create a new BrowserContext with specified options.
         See `Browser, Context and Page` for more information about BrowserContext.
@@ -333,65 +347,89 @@ class PlaywrightState(LibraryComponent):
         Returns a stable identifier for the created context
         that can be used in `Switch Context`.
 
-        ``acceptDownloads`` <bool> Whether to automatically downloadall the attachments.
+        ``acceptDownloads`` Whether to automatically downloads all the attachments.
         Defaults to False where all the downloads are canceled.
 
-        ``ignoreHTTPSErrors`` <bool> Whether to ignore HTTPS errors during navigation.
+        ``ignoreHTTPSErrors`` Whether to ignore HTTPS errors during navigation.
         Defaults to False.
 
-        ``bypassCSP`` <bool> Toggles bypassing page's Content-Security-Policy. Defaults to False.
+        ``bypassCSP`` Toggles bypassing page's Content-Security-Policy. Defaults to False.
 
-        ``viewport`` <dict> Sets a consistent viewport for each page.
+        ``viewport`` Sets a consistent viewport for each page.
         Defaults to an ``{'width': 1280, 'height': 720}`` viewport.
         Value of ``viewport`` can be a dict or a string
         representation of a dictionary.
 
-        ``userAgent`` <str> Specific user agent to use in this context.
+        ``userAgent`` Specific user agent to use in this context.
 
-        ``deviceScaleFactor`` <float> Specify device scale factor
+        ``deviceScaleFactor`` Specify device scale factor
         (can be thought of as dpr). Defaults to 1.
 
-        ``isMobile`` <bool> Whether the meta viewport tag is taken into account
+        ``isMobile`` Whether the meta viewport tag is taken into account
         and touch events are enabled. Defaults to False. Not supported in Firefox.
 
-        ``hasTouch`` <bool> Specifies if viewport supports touch events. Defaults to False.
+        ``hasTouch`` Specifies if viewport supports touch events. Defaults to False.
 
-        ``javaScriptEnabled`` <bool> Whether or not to enable JavaScript in the context.
+        ``javaScriptEnabled`` Whether or not to enable JavaScript in the context.
         Defaults to True.
 
-        ``timezoneId`` <str> Changes the timezone of the context.
+        ``timezoneId`` Changes the timezone of the context.
         See [https://source.chromium.org/chromium/chromium/src/+/master:third_party/icu/source/data/misc/metaZones.txt | ICU’s metaZones.txt]
         for a list of supported timezone IDs.
 
-        ``geolocation`` <dict> Sets the geolocation. No location is set be default.
-        - ``latitude`` <number> Latitude between -90 and 90. **Required**
-        - ``longitude`` <number> Longitude between -180 and 180. **Required**
+        ``geolocation`` Sets the geolocation. No location is set by default.
+        - ``latitude`` <number> Latitude between -90 and 90.
+        - ``longitude`` <number> Longitude between -180 and 180.
         - ``accuracy`` Optional <number> Non-negative accuracy value. Defaults to 0.
         Example usage: ``{'latitude': 59.95, 'longitude': 30.31667}``
 
-        ``locale`` <str> Specify user locale, for example ``en-GB``, ``de-DE``, etc.
+        ``locale`` Specify user locale, for example ``en-GB``, ``de-DE``, etc.
         Locale will affect ``navigator.language`` value, ``Accept-Language`` request header value
         as well as number and date formatting rules.
 
-        ``permissions`` <list<str>> A list of permissions to grant to all pages in this context.
+        ``permissions`` A list of permissions to grant to all pages in this context.
         See [https://github.com/microsoft/playwright/blob/master/docs/api.md#browsercontextgrantpermissionspermissions-options | grantPermissions]
         for more details.
 
-        ``extraHTTPHeaders`` <dict[str, str]> A dictionary containing additional HTTP headers
+        ``extraHTTPHeaders`` A dictionary containing additional HTTP headers
         to be sent with every request. All header values must be strings.
 
-        ``offline`` <bool> Whether to emulate network being offline. Defaults to False.
+        ``offline`` Whether to emulate network being offline. Defaults to False.
 
-        ``httpCredentials`` <Dict<str, str>> Credentials for HTTP authentication.
+        ``httpCredentials`` Credentials for HTTP authentication.
         - example: ``{'username': 'admin', 'password': '123456'}``
-        - ``username`` <str>
-        - ``password`` <str>
+        - ``username``
+        - ``password``
 
-        ``colorScheme`` <dark|light|no-preference> Emulates 'prefers-colors-scheme'
+        ``colorScheme`` Emulates 'prefers-colors-scheme'
         media feature, supported values are 'light', 'dark', 'no-preference'.
         See [https://github.com/microsoft/playwright/blob/master/docs/api.md#pageemulatemediaoptions|emulateMedia(options)]
         for more details. Defaults to ``light``.
 
+        ``proxy`` Network proxy settings to use with this context.
+        Note that browser needs to be launched with the global proxy for this option to work.
+        If all contexts override the proxy, global proxy will be never used and can be any string
+
+        ``videosPath`` Enables video recording for all pages to videosPath
+        folder. If not specified, videos are not recorded.
+
+        ``videoSize`` Specifies dimensions of the automatically recorded
+        video. Can only be used if videosPath is set. If not specified the size will
+        be equal to viewport. If viewport is not configured explicitly the video size
+        defaults to 1280x720. Actual picture of the page will be scaled down if
+        necessary to fit specified size.
+        - Example {"width": 1280, "height": 720}
+
+        ``defaultBrowserType`` If no browser is open and `New Context` opens a new browser
+        with defaults, it now uses this setting.
+        Very useful together with `Get Device` keyword:
+
+        Example:
+        | Test an iPhone
+        |     ${device}=    `Get Device`    iPhone X
+        |     `New Context`    &{device}        # unpacking here with &
+        |     `New Page`    http://example.com
+
         A BrowserContext is the Playwright object that controls a single browser profile.
         Within a context caches and cookies are shared.
         See [https://github.com/microsoft/playwright/blob/master/docs/api.md#browsernewcontextoptions|Playwright browser.newContext]
@@ -400,25 +438,31 @@ class PlaywrightState(LibraryComponent):
         If there's no open Browser this keyword will open one. Does not create pages.
         """
         params = locals_to_params(locals())
-        if "geolocation" in params:
-            location = params["geolocation"]
-            latitude = location["latitude"]
-            location["latitude"] = float(latitude)
-            longitude = location["longitude"]
-            location["longitude"] = float(longitude)
-            accuracy = location.get("accuracy")
-            if accuracy:
-                location["accuracy"] = float(accuracy)
+        params = convert_typed_dict(self.new_context.__annotations__, params)
+        if not videosPath:
+            params.pop("videoSize", None)
         options = json.dumps(params, default=str)
         logger.info(options)
         with self.playwright.grpc_channel() as stub:
             response = stub.NewContext(
-                Request().Context(rawOptions=options, hideRfBrowser=hideRfBrowser)
+                Request().Context(
+                    rawOptions=options,
+                    hideRfBrowser=hideRfBrowser,
+                    defaultTimeout=int(self.timeout),
+                )
             )
-            logger.info(response.log)
-            return response.body
+        logger.info(response.log)
+        self.context_cache.add(response.body, self._get_video_size(params))
+        return response.body
 
-    @keyword(tags=["Setter", "BrowserControl"])
+    def _get_video_size(self, params: dict) -> dict:
+        if "videoSize" in params:
+            return params["videoSize"]
+        if "viewport" in params:
+            return params["viewport"]
+        return {"width": 1280, "height": 720}
+
+    @keyword(tags=("Setter", "BrowserControl"))
     def new_page(self, url: Optional[str] = None) -> str:
         """Open a new Page. A Page is the Playwright equivalent to a tab.
         See `Browser, Context and Page` for more information about Page concept.
@@ -427,15 +471,34 @@ class PlaywrightState(LibraryComponent):
         When a `New Page` is called without an open browser, `New Browser`
         and `New Context` are executed with default values first.
 
-        ``url`` <str> If specified it will open the new page to the specified URL.
+        ``url`` If specified it will open the new page to the specified URL.
 
         """
         with self.playwright.grpc_channel() as stub:
-            response = stub.NewPage(Request().Url(url=url))
-            logger.info(response.log)
-            return response.body
+            response = stub.NewPage(
+                Request().Url(url=url, defaultTimeout=int(self.timeout))
+            )
+        logger.info(response.log)
+        self._embed_video(json.loads(response.video))
+        return response.body
 
-    @keyword(tags=["Getter", "BrowserControl", ""])
+    def _embed_video(self, video: dict):
+        if not video.get("video_path"):
+            logger.debug("Video is not enabled.")
+            return
+        relative_path = get_link_path(video.get("video_path"), self.outputdir)
+        video_size = self.context_cache.get(video["contextUuid"])
+        video_width = video_size["width"]
+        video_height = video_size["height"]
+        video_type = relative_path.split(".")[1]
+        logger.info(
+            '</td></tr><tr><td colspan="3">'
+            f'<video width="{video_width}" height="{video_height}" controls>'
+            f'<source src="{relative_path}" type="video/{video_type}"></video>',
+            html=True,
+        )
+
+    @keyword(tags=("Getter", "BrowserControl"))
     @with_assertion_polling
     def get_browser_catalog(
         self,
@@ -517,26 +580,26 @@ class PlaywrightState(LibraryComponent):
                 parsed, assertion_operator, assertion_expected, "Browser Catalog "
             )
 
-    @keyword(tags=["Setter", "BrowserControl"])
+    @keyword(tags=("Setter", "BrowserControl"))
     def switch_browser(self, id: str) -> str:
         """Switches the currently active Browser to another open Browser.
         Returns a stable identifier for the previous browser.
         See `Browser, Context and Page` for more information about Browser and related concepts.
 
-        ``id`` <str> Id of the browser to be changed to. Starting at 0. **Required**
+        ``id`` Id of the browser to be changed to. Starting at 0.
         """
         with self.playwright.grpc_channel() as stub:
             response = stub.SwitchBrowser(Request().Index(index=id))
             logger.info(response.log)
             return response.body
 
-    @keyword(tags=["Setter", "BrowserControl"])
+    @keyword(tags=("Setter", "BrowserControl"))
     def switch_context(self, id: str, browser: str = "CURRENT") -> str:
         """Switches the active BrowserContext to another open context.
         Returns a stable identifier for the previous context.
         See `Browser, Context and Page` for more information about Context and related concepts.
 
-        ``id`` <str> Id of the context to be changed to. Randomly generated UUID. **Required**
+        ``id`` Id of the context to be changed to. Randomly generated UUID.
 
         ``browser`` < ``CURRENT`` | str> Switch context in specified browser. If value is not "CURRENT"
         it should be an the id of the browser where to switch context.
@@ -547,7 +610,7 @@ class PlaywrightState(LibraryComponent):
             logger.info(response.log)
             return response.body
 
-    @keyword(tags=["Setter", "BrowserControl"])
+    @keyword(tags=("Setter", "BrowserControl"))
     def switch_page(
         self, id: str, context: str = "CURRENT", browser: str = "CURRENT"
     ) -> str:
@@ -561,7 +624,7 @@ class PlaywrightState(LibraryComponent):
         exists before library timeout. See `Set Browser Timeout` for how to change the timeout.
 
         With ``CURRENT`` you can get the ``id`` of the "CURRENT" page
-        **Required**
+
 
         ``context`` < ``CURRENT`` | str> Switch page in specified context. If value is not "CURRENT"
         it should be the id of the context where to switch page.
@@ -583,13 +646,13 @@ class PlaywrightState(LibraryComponent):
             logger.info(response.log)
             return response.body
 
-    @keyword(tags=["Getter", "BrowserControl"])
+    @keyword(tags=("Getter", "BrowserControl"))
     def get_browser_ids(self, browser: SelectionType = SelectionType.ALL) -> List:
         """Returns a list of ids from open browsers.
         See `Browser, Context and Page` for more information about Browser and related concepts.
 
 
-        ``browser`` < ``ALL`` | ``ACTIVE`` | ``CURRENT`` > Defaults to ``ALL``
+        ``browser`` Defaults to ``ALL``
         - ``ALL`` Returns all ids as a list.
         - ``ACTIVE`` or ``CURRENT`` Returns the id of the currently active browser as list.
 
@@ -603,7 +666,7 @@ class PlaywrightState(LibraryComponent):
             return [browser["id"] for browser in self.get_browser_catalog()]
         return []
 
-    @keyword(tags=["Getter", "BrowserControl"])
+    @keyword(tags=("Getter", "BrowserControl"))
     def get_context_ids(
         self,
         context: SelectionType = SelectionType.ALL,
@@ -613,11 +676,11 @@ class PlaywrightState(LibraryComponent):
         See `Browser, Context and Page` for more information about Context and related concepts.
 
 
-        ``context`` < ``ALL`` | ``ACTIVE`` > Defaults to ``ALL``
+        ``context`` Defaults to ``ALL``
         - ``ALL`` Returns all context ids as a list.
         - ``ACTIVE`` Returns the id of the active context as a list.
 
-        ``browser`` < ``ALL`` | ``ACTIVE`` > Defaults to ``ALL``
+        ``browser`` Defaults to ``ALL``
         - ``ALL`` context ids from all open browsers shall be fetched.
         - ``ACTIVE`` only context ids from the active browser shall be fetched.
 
@@ -646,7 +709,7 @@ class PlaywrightState(LibraryComponent):
                 return context_ids
         return []
 
-    @keyword(tags=["Getter", "BrowserControl"])
+    @keyword(tags=("Getter", "BrowserControl"))
     def get_page_ids(
         self,
         page: SelectionType = SelectionType.ALL,
@@ -657,15 +720,15 @@ class PlaywrightState(LibraryComponent):
         See `Browser, Context and Page` for more information about Page and related concepts.
 
 
-        ``page`` < ``ALL`` | ``ACTIVE`` >
+        ``page``
         - ``ALL`` Returns all page ids as a list.
         - ``ACTIVE`` Returns the id of the active page as a list.
 
-        ``context`` < ``ALL`` | ``ACTIVE`` >
+        ``context``
         - ``ALL`` page ids from all contexts shall be fetched.
         - ``ACTIVE`` only page ids from the active context shall be fetched.
 
-        ``browser`` < ``ALL`` | ``ACTIVE`` >
+        ``browser``
         - ``ALL`` page ids from all open browsers shall be fetched.
         - ``ACTIVE`` only page ids from the active browser shall be fetched.
 
